const express = require('express');
const Web3 = require('web3');
const fs = require('fs');

// Initialize Express app
const app = express();

// Initialize Web3 with your RPC URL
const web3 = new Web3('https://rpc.siftoscan.com');

// Read the compiled contract bytecode and ABI
const bytecode = fs.readFileSync('SIFTO_bytecode.txt', 'utf8');
const abi = [{"type":"constructor","payable":false,"inputs":[{"type":"uint256","name":"initialSupply"}]},{"type":"error","name":"ERC20InsufficientAllowance","inputs":[{"type":"address","name":"spender"},{"type":"uint256","name":"allowance"},{"type":"uint256","name":"needed"}]},{"type":"error","name":"ERC20InsufficientBalance","inputs":[{"type":"address","name":"sender"},{"type":"uint256","name":"balance"},{"type":"uint256","name":"needed"}]},{"type":"error","name":"ERC20InvalidApprover","inputs":[{"type":"address","name":"approver"}]},{"type":"error","name":"ERC20InvalidReceiver","inputs":[{"type":"address","name":"receiver"}]},{"type":"error","name":"ERC20InvalidSender","inputs":[{"type":"address","name":"sender"}]},{"type":"error","name":"ERC20InvalidSpender","inputs":[{"type":"address","name":"spender"}]},{"type":"event","anonymous":false,"name":"Approval","inputs":[{"type":"address","name":"owner","indexed":true},{"type":"address","name":"spender","indexed":true},{"type":"uint256","name":"value","indexed":false}]},{"type":"event","anonymous":false,"name":"Transfer","inputs":[{"type":"address","name":"from","indexed":true},{"type":"address","name":"to","indexed":true},{"type":"uint256","name":"value","indexed":false}]},{"type":"function","name":"allowance","constant":true,"stateMutability":"view","payable":false,"inputs":[{"type":"address","name":"owner"},{"type":"address","name":"spender"}],"outputs":[{"type":"uint256"}]},{"type":"function","name":"approve","constant":false,"payable":false,"inputs":[{"type":"address","name":"spender"},{"type":"uint256","name":"value"}],"outputs":[{"type":"bool"}]},{"type":"function","name":"balanceOf","constant":true,"stateMutability":"view","payable":false,"inputs":[{"type":"address","name":"account"}],"outputs":[{"type":"uint256"}]},{"type":"function","name":"checkBalance","constant":true,"stateMutability":"view","payable":false,"inputs":[{"type":"address","name":"account"}],"outputs":[{"type":"uint256"}]},{"type":"function","name":"decimals","constant":true,"stateMutability":"view","payable":false,"inputs":[],"outputs":[{"type":"uint8"}]},{"type":"function","name":"name","constant":true,"stateMutability":"view","payable":false,"inputs":[],"outputs":[{"type":"string"}]},{"type":"function","name":"symbol","constant":true,"stateMutability":"view","payable":false,"inputs":[],"outputs":[{"type":"string"}]},{"type":"function","name":"totalSupply","constant":true,"stateMutability":"view","payable":false,"inputs":[],"outputs":[{"type":"uint256"}]},{"type":"function","name":"transfer","constant":false,"payable":false,"inputs":[{"type":"address","name":"to"},{"type":"uint256","name":"value"}],"outputs":[{"type":"bool"}]},{"type":"function","name":"transferFrom","constant":false,"payable":false,"inputs":[{"type":"address","name":"from"},{"type":"address","name":"to"},{"type":"uint256","name":"value"}],"outputs":[{"type":"bool"}]},{"type":"function","name":"transferTokens","constant":false,"payable":false,"inputs":[{"type":"address","name":"recipient"},{"type":"uint256","name":"amount"}],"outputs":[{"type":"bool"}]}];

// Endpoint for user registration
app.get('/register', async (req, res) => {
    try {
        // Deploy contract for the user with an initial supply of tokens (e.g., 100)
        const contractAddress = await deployContract(100);
        res.json({ contractAddress });
    } catch (error) {
        console.error('Error deploying contract:', error);
        res.status(500).json({ error: 'Failed to deploy contract' });
    }
});

// Function to deploy the contract
async function deployContract(initialSupply) {
    const accounts = await web3.eth.getAccounts();
    const deployer = accounts[0]; // You may want to manage different deployers for each user

    const contract = new web3.eth.Contract(abi);

    // Deploy the contract with constructor arguments
    const deployedContract = await contract.deploy({
        data: bytecode, // Ensure bytecode has '0x' prefix
        arguments: [initialSupply] // Pass the initial supply as the constructor argument
    }).send({
        from: deployer,
        gas: '5000000' // Adjust gas limit as needed
    });

    // Return the deployed contract address
    return deployedContract.options.address;
}




app.get('/transfer', async (req, res) => {
    const { amount, recipientAddress, contractAddress } = req.query; // Access amount, recipientAddress, and contractAddress from the query parameters

     // Your ABI definition here

    try {
        // Initialize contract instance with ABI and contract address
        const contractInstance = new web3.eth.Contract(abi, contractAddress);

        // Send a transaction to transfer tokens
        const transaction = await contractInstance.methods.transfer(recipientAddress, amount)
            .send({ from: '0x10245f281C35ecd3aA5F8BF9e87e62490C5bb2D8', gas: '500000' });

        res.json({ transactionHash: transaction.transactionHash });
    } catch (error) {
        console.error('Error transferring tokens:', error);
        res.status(500).json({ error: 'Failed to transfer tokens' });
    }
});



// Start the server
const PORT = process.env.PORT || 3400;
app.listen(PORT, () => {
    console.log(`Server is running on port ${PORT}`);
});

